<?php

namespace EventPortal\Domain\ActionList\Models;

use EventPortal\Domain\User\Models\User;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Support\Arr;

class Task extends Model
{
    /** @var array */
    protected $fillable = [
        'label',
        'name',
        'completed_at',
        'user_id',
        'required',
        'action',
    ];

    /** @var array */
    protected $casts = [
        'action' => 'json',
    ];

    public function actionList(): BelongsTo
    {
        return $this->belongsTo(ActionList::class);
    }

    public function getIsCompleteAttribute(): bool
    {
        return Arr::get($this->attributes, 'completed_at') !== null;
    }

    public function getCompletedByAttribute(): ?User
    {
        return User::whereId(Arr::get($this->attributes, 'user_id'))->first();
    }

    public function toggle(User $user): void
    {
        if ($this->is_complete) {
            $this->setAsIncomplete();

            return;
        }

        $this->setAsComplete($user);
    }

    public function setAsComplete(User $user): void
    {
        $this->update([
            'completed_at' => now(),
            'user_id' => $user->id,
        ]);
    }

    public function setAsIncomplete(): void
    {
        $this->update([
            'completed_at' => null,
            'user_id' => null,
        ]);
    }

    public function scopeCompleted(Builder $query): Builder
    {
        return $query->whereNotNull('completed_at');
    }

    public function scopeIncomplete(Builder $query): Builder
    {
        return $query->whereNull('completed_at');
    }
}
