<?php

namespace EventPortal\Http\Controllers\Delegate\Event;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\ValueObjects\PageHeader;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Util\DelegateRender;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Inertia\Response;

class ContactController
{
    public function index(Event $event): Response
    {
        $form = Form::make('contact.create')->context(Auth::user(), $event)->render();

        return DelegateRender::component('Delegate/Contact/Index')
            ->key('contact.index')
            ->withForm($form)
            ->withPageHeader(PageHeader::make()->title('Contact'))
            ->get();
    }

    public function store(Request $request, Event $event): RedirectResponse
    {
        Form::make('contact.create')
            ->model($event)
            ->validate($request->input())
            ->handle($request->input());

        toast()->success(trans('event.contact.message-sent'));

        return redirect()->route('delegate.landing', ['event' => $event->slug]);
    }
}
